package com.guwan.backend.security;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.guwan.backend.common.Result;
import com.guwan.backend.util.JwtUtil;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

@Slf4j
@Component
@RequiredArgsConstructor
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private final JwtUtil jwtUtil;
    private final UserDetailsServiceImpl userDetailsService;
    private final ObjectMapper objectMapper;

    // 不需要验证的路径
    private static final List<String> PERMIT_PATHS = Arrays.asList(
            "/demo/**",
            "/api/user/register",
            "/api/user/login",
            "/api/user/register/email",
            "/api/user/register/phone",
            "/api/user/email/code",
            "/api/user/phone/code"
    );

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
            throws ServletException, IOException {
        
        // 检查是否是允许的路径
        String path = request.getServletPath();
        if (isPermitPath(path)) {
            chain.doFilter(request, response);
            return;
        }

        try {
            String authHeader = request.getHeader("Authorization");
            if (authHeader == null || !authHeader.startsWith("Bearer ")) {
                handleAuthenticationError(response, "未登录");
                return;
            }

            String jwt = authHeader.substring(7);
            if (!jwtUtil.validateToken(jwt)) {
                handleAuthenticationError(response, "token已过期或无效");
                return;
            }

            Long userId = jwtUtil.getUserIdFromToken(jwt);
            if (SecurityContextHolder.getContext().getAuthentication() == null) {
                UserDetails userDetails = userDetailsService.loadUserById(userId);
                
                UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(
                        userDetails, null, userDetails.getAuthorities());
                authentication.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));

                SecurityContextHolder.getContext().setAuthentication(authentication);
            }

            chain.doFilter(request, response);
        } catch (Exception e) {
            log.error("认证过程发生错误", e);
            handleAuthenticationError(response, "认证失败: " + e.getMessage());
        }
    }

    private boolean isPermitPath(String path) {
        return PERMIT_PATHS.stream().anyMatch(pattern -> 
            pattern.endsWith("/**") 
                ? path.startsWith(pattern.substring(0, pattern.length() - 3))
                : path.equals(pattern)
        );
    }

    private void handleAuthenticationError(HttpServletResponse response, String message) throws IOException {
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding("UTF-8");
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
        
        Result<?> result = Result.error(message);
        response.getWriter().write(objectMapper.writeValueAsString(result));
    }

    @Override
    protected boolean shouldNotFilter(HttpServletRequest request) {
        String path = request.getServletPath();
        // 静态资源不过滤
        return path.startsWith("/static/") || 
               path.startsWith("/public/") || 
               path.startsWith("/error");
    }
} 